@@ -25,6 +25,8 @@
 #include <libgda/gda-intl.h>
 #include <libgda/gda-data-model.h>
 #include <libgda/gda-log.h>
+#include <libgda/gda-xml-database.h>
+#include <string.h>
 
 #define PARENT_TYPE G_TYPE_OBJECT
 #define CLASS(model) (GDA_DATA_MODEL_CLASS (G_OBJECT_GET_CLASS (model)))
@@ -45,6 +47,9 @@ static void gda_data_model_finalize   (GObject *object);
 
 enum {
 	CHANGED,
+	ROW_INSERTED,
+	ROW_UPDATED,
+	ROW_REMOVED,
 	BEGIN_EDIT,
 	CANCEL_EDIT,
 	END_EDIT,
@@ -73,6 +78,30 @@ gda_data_model_class_init (GdaDataModelClass *klass)
                               NULL, NULL,
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE, 0);
+	gda_data_model_signals[ROW_INSERTED] =
+		g_signal_new ("row_inserted",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdaDataModelClass, row_inserted),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__INT,
+                              G_TYPE_NONE, 1, G_TYPE_INT);
+	gda_data_model_signals[ROW_UPDATED] =
+		g_signal_new ("row_updated",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdaDataModelClass, row_updated),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__INT,
+                              G_TYPE_NONE, 1, G_TYPE_INT);
+	gda_data_model_signals[ROW_REMOVED] =
+		g_signal_new ("row_removed",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdaDataModelClass, row_removed),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__INT,
+                              G_TYPE_NONE, 1, G_TYPE_INT);
 	gda_data_model_signals[BEGIN_EDIT] =
 		g_signal_new ("begin_edit",
                               G_TYPE_FROM_CLASS (object_class),
@@ -144,9 +173,11 @@ gda_data_model_finalize (GObject *object)
 	/* free memory */
 	g_hash_table_foreach (model->priv->column_titles, free_hash_string, NULL);
 	g_hash_table_destroy (model->priv->column_titles);
+	model->priv->column_titles = NULL;
 
 	g_free (model->priv->cmd_text);
 	model->priv->cmd_text = NULL;
+
 	g_free (model->priv);
 	model->priv = NULL;
 
@@ -200,6 +231,54 @@ gda_data_model_changed (GdaDataModel *model)
 	}
 }
 
+/**
+ * gda_data_model_row_inserted
+ * model: a #GdaDataModel object.
+ */
+void
+gda_data_model_row_inserted (GdaDataModel *model, gint row)
+{
+	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+
+	if (model->priv->notify_changes) {
+		g_signal_emit (G_OBJECT (model),
+			       gda_data_model_signals[ROW_INSERTED],
+			       0, row);
+	}
+}
+
+/**
+ * gda_data_model_row_updated
+ * model: a #GdaDataModel object.
+ */
+void
+gda_data_model_row_updated (GdaDataModel *model, gint row)
+{
+	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+
+	if (model->priv->notify_changes) {
+		g_signal_emit (G_OBJECT (model),
+			       gda_data_model_signals[ROW_UPDATED],
+			       0, row);
+	}
+}
+
+/**
+ * gda_data_model_row_removed
+ * model: a #GdaDataModel object.
+ */
+void
+gda_data_model_row_removed (GdaDataModel *model, gint row)
+{
+	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+
+	if (model->priv->notify_changes) {
+		g_signal_emit (G_OBJECT (model),
+			       gda_data_model_signals[ROW_REMOVED],
+			       0, row);
+	}
+}
+
 /**
  * gda_data_model_freeze
  * @model: a #GdaDataModel object.
@@ -281,10 +360,12 @@ gda_data_model_describe_column (GdaDataModel *model, gint col)
 	fa = CLASS (model)->describe_column (model, col);
 	if (!fa) {
 		const GdaValue *value;
+
 		/* we generate a basic FieldAttributes structure */
 		fa = gda_field_attributes_new ();
 		gda_field_attributes_set_defined_size (fa, 0);
-		gda_field_attributes_set_name (fa, gda_data_model_get_column_title (model, col));
+		gda_field_attributes_set_name (fa, g_hash_table_lookup (model->priv->column_titles,
+									GINT_TO_POINTER (col)));
 		gda_field_attributes_set_scale (fa, 0);
 		value = gda_data_model_get_value_at (model, col, 0);
 		if (value == NULL)
@@ -317,8 +398,20 @@ gda_data_model_get_column_title (GdaDataModel *model, gint col)
 	if (col < n_cols && col >= 0){
 		title = g_hash_table_lookup (model->priv->column_titles,
 					     GINT_TO_POINTER (col));
-		if (title == NULL)
-			title = "";
+		if (title == NULL) {
+			GdaFieldAttributes *fa;
+
+			fa = gda_data_model_describe_column (model, col);
+			if (fa) {
+				gda_data_model_set_column_title (model, col, title);
+				gda_field_attributes_free (fa);
+
+				return g_hash_table_lookup (model->priv->column_titles,
+							    GINT_TO_POINTER (col));
+			}
+			else
+				title = "";
+		}
 	}
 	else
 		title = "";
@@ -341,11 +434,15 @@ gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *tit
 
 	n_cols = gda_data_model_get_n_columns (model);
 	if (col >= 0 && col < n_cols) {
-		gpointer value;
+		gpointer key, value;
 
-		value = g_hash_table_lookup (model->priv->column_titles,
+		if (g_hash_table_lookup_extended (model->priv->column_titles,
+						  GINT_TO_POINTER (col),
+						  &key, &value)) {
+			g_hash_table_remove (model->priv->column_titles,
 					     GINT_TO_POINTER (col));
-		g_free (value);
+			g_free (value);
+		}
 
 		g_hash_table_insert (model->priv->column_titles, 
 				     GINT_TO_POINTER (col), g_strdup (title));
@@ -537,7 +634,7 @@ gda_data_model_foreach (GdaDataModel *model,
 	cols = gda_data_model_get_n_columns (model);
 
 	for (r = 0; r < rows; r++) {
-		row = gda_row_new (cols);
+		row = gda_row_new (model, cols);
 		for (c = 0; c < cols; c++) {
 			GdaValue *value;
 			value = gda_value_copy (gda_data_model_get_value_at (model, c, r));
@@ -707,7 +804,119 @@ gda_data_model_to_tab_separated (GdaDataModel *model)
 gchar *
 gda_data_model_to_xml (GdaDataModel *model, gboolean standalone)
 {
-	return NULL;
+	xmlDocPtr xml_doc;
+	xmlNodePtr xml_root, xml_node, xml_tables;
+	gchar *xml = NULL;
+	xmlChar *xml_contents;
+	gint size;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
+
+	xml_node = gda_data_model_to_xml_node (model, "exported_model");
+	if (standalone) {
+		xml_doc = xmlNewDoc ("1.0");
+		xml_root = xmlNewDocNode (xml_doc, NULL, "database", NULL);
+		xmlDocSetRootElement (xml_doc, xml_root);
+
+		xml_tables = xmlNewChild (xml_root, NULL, "tables", NULL);
+		xmlAddChild (xml_tables, xml_node);
+
+		xmlDocDumpMemory (xml_doc, &xml_contents, &size);
+		xmlFreeDoc (xml_doc);
+
+		xml = g_strdup (xml_contents);
+	}
+
+	return xml;
+}
+
+static void
+xml_set_boolean (xmlNodePtr node, const gchar *name, gboolean value)
+{
+	xmlSetProp (node, name, value ? "1" : "0");
+}
+
+static void
+xml_set_int (xmlNodePtr node, const gchar *name, gint value)
+{
+	char s[80];
+
+	sprintf (s, "%d", value);
+	xmlSetProp (node, name, s);
+}
+
+/**
+ * gda_data_model_to_xml_node
+ * @model: A #GdaDataModel object.
+ * @name: Name to use for the XML resulting table.
+ *
+ * Convert a #GdaDataModel into a xmlNodePtr (as used in libxml).
+ *
+ * Returns: a xmlNodePtr representing the whole data model.
+ */
+xmlNodePtr
+gda_data_model_to_xml_node (GdaDataModel *model, const gchar *name)
+{
+	xmlNodePtr *node;
+	gint cols, rows, i;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
+
+	node = xmlNewNode (NULL, "table");
+	if (name != NULL)
+		xmlSetProp (node, "name", name);
+
+	/* set the table structure */
+	rows = gda_data_model_get_n_rows (model);
+	cols = gda_data_model_get_n_columns (model);
+	for (i = 0; i < cols; i++) {
+		GdaFieldAttributes *fa;
+		xmlNodePtr field;
+
+		fa = gda_data_model_describe_column (model, i);
+		if (!fa) {
+			xmlFreeNode (node);
+			return NULL;
+		}
+
+		field = xmlNewChild (node, NULL, "field", NULL);
+		xmlSetProp (field, "name", gda_field_attributes_get_name (fa));
+		xmlSetProp (field, "caption", gda_field_attributes_get_caption (fa));
+		xmlSetProp (field, "gdatype", gda_type_to_string (gda_field_attributes_get_gdatype (fa)));
+		xml_set_int (field, "size", gda_field_attributes_get_defined_size (fa));
+		xml_set_int (field, "scale", gda_field_attributes_get_scale (fa));
+		xml_set_boolean (field, "pkey", gda_field_attributes_get_primary_key (fa));
+		xml_set_boolean (field, "unique", gda_field_attributes_get_unique_key (fa));
+		xml_set_boolean (field, "isnull", gda_field_attributes_get_allow_null (fa));
+		xml_set_boolean (field, "auto_increment", gda_field_attributes_get_auto_increment (fa));
+		xmlSetProp (field, "references", gda_field_attributes_get_references (fa));
+		xml_set_int (field, "position", i);
+	}
+	
+	/* add the model data to the XML output */
+	if (rows > 0) {
+		xmlNodePtr row, data, field;
+		gint r, c;
+
+		data = xmlNewChild (node, NULL, "data", NULL);
+		for (r = 0; r < rows; r++) {
+			row = xmlNewChild (data, NULL, "row", NULL);
+			xml_set_int (row, "position", r);
+			for (c = 0; c < cols; c++) {
+				const GdaValue *value;
+				gchar *str;
+
+				value = gda_data_model_get_value_at (model, c, r);
+				str = gda_value_stringify (value);
+				field = xmlNewChild (row, NULL, "field", str);
+				xml_set_int (field, "position", c);
+
+				g_free (str);
+			}
+		}
+	}
+
+	return node;
 }
 
 /**
