@@ -38,7 +38,6 @@ G_BEGIN_DECLS
 #define GDA_IS_DATA_MODEL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, GDA_TYPE_DATA_MODEL))
 #define GDA_IS_DATA_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GDA_TYPE_DATA_MODEL))
 
-typedef struct _GdaDataModel        GdaDataModel;
 typedef struct _GdaDataModelClass   GdaDataModelClass;
 typedef struct _GdaDataModelPrivate GdaDataModelPrivate;
 
@@ -52,6 +51,9 @@ struct _GdaDataModelClass {
 
 	/* signals */
 	void (* changed) (GdaDataModel *model);
+	void (* row_inserted) (GdaDataModel *model, gint row);
+	void (* row_updated) (GdaDataModel *model, gint row);
+	void (* row_removed) (GdaDataModel *model, gint row);
 
 	void (* begin_edit) (GdaDataModel *model);
 	void (* cancel_edit) (GdaDataModel *model);
@@ -73,6 +75,9 @@ struct _GdaDataModelClass {
 GType               gda_data_model_get_type (void);
 
 void                gda_data_model_changed (GdaDataModel *model);
+void                gda_data_model_row_inserted (GdaDataModel *model, gint row);
+void                gda_data_model_row_updated (GdaDataModel *model, gint row);
+void                gda_data_model_row_removed (GdaDataModel *model, gint row);
 void                gda_data_model_freeze (GdaDataModel *model);
 void                gda_data_model_thaw (GdaDataModel *model);
 
